#grt4p.tcl---GRT 4P module control frame


set frame
set frametitle "GRT 4P Module Setup & Monitor"
set icontitle "GRT4P"

set modulename  grt4p
set moduletype  "grt4p"

set defaultmodulelist [list 1 2 3 4 5 6 7 8]

set modulelist {???}
set moduleindex 0

set maxmodule 8

set maxmodule [inform vmecontrol inquire [set modulename]count]
set modulelist {}
for {set i 1} {$i <= $maxmodule} {incr i} {lappend modulelist $i}

set module [lindex $modulelist $moduleindex]
set allmodules 0
set allchannels 0

foreach ch {A B C D} {
  set w2r(DPP[set ch]CSR) Ch$ch.DPP_CSR
  set w2r(DPP[set ch]WPTR) Ch$ch.DPP_Wptr
  set w2r(DPP[set ch]RPTR) Ch$ch.DPP_Rptr
  for {set i 4} {$i <= 6} {incr i} {
    set w2r(DPP$ch$i) Ch$ch.DPP_Reg$i
  }
}

foreach ch {A B C D} {
  for {set i 3} {$i <= 4} {incr i} {
    set w2r(ADC$ch$i) Ch$ch.ADC_Spare$i
  }
  set w2r(ADC[set ch]PRETRIG) Ch$ch.ADC_PreTrig
  set w2r(ADC[set ch]HEADER)  Ch$ch.ADC_Header
  set w2r(ADC[set ch]TRIGGER) Ch$ch.ADC_Trigger
  set w2r(ADC[set ch]CTRL)    Ch$ch.ADC_Ctrl
}


loadsource common.vme


proc click-select-module {v} {
    global modulename module CurrentVMEServer
    set module $v
    click-redisplay
}
  
proc click-software {} {
   global modulename
   inform [set modulename]sftwre start
}

proc click-registers {} {}

proc click-widget {w v} {
    global w2r
    global modulename module
    global allmodules allchannels
    global CurrentVMEServer

    set register $w2r($w)
    if {$allchannels == 1} then {set register [string range $register 0 1]?[string range $register 3 end]}
    set value $v

    if {$allmodules == 0} then {
        write-vme-registers $modulename#$module.$register $value
    } else {
        set CurrentVMEServer [inform vmecontrol inquire Role]
        write-vme-registers $modulename#\[?,??\].$register $value
        set CurrentVMEServer [inform vmecontrol VMEmodule2server $modulename#$module]
    }

    click-redisplay
}

proc read_fifo {ch} {
    global modulename module
    global EGallow

    set register "Ch$ch.DataFiFo"
    set EGallow 0x2000a
    set value [read-vme-register $modulename#$module.$register]
    set EGallow 0xffffffff
    if {$value == 0x2000a} then {set value "BERR"}
    :modify DATA$ch :value $value
}

proc click-redisplay {} {
    global modulename module
    global EGallow EGreport CurrentVMEServer
    global w2r

    :modify MODULE-NUM :value $module
    set CurrentVMEServer [inform vmecontrol VMEmodule2server $modulename#$module]

   set EGreport -1
   set-busy
   foreach w [array names w2r] {
       set register $w2r($w)
       set EGallow 0x2000a
       set value [read-vme-register $modulename#$module.$register]
       set EGallow 0xffffffff
       if {$value == 0x2000a} then {set value "BERR"}
       :modify $w :value $value
   }
   clear-busy
   set EGreport 0
}

proc click arg {
   global debug actionlist allmodules allchannels
   if {$debug} then {puts insert-debug "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   set i [lindex $arg 4]
   if {$c==":quit"} then {click-quit; return}
   switch -glob $w {

      MODULE-SEL      -
      MODULE-NUM      {click-select-module $v}
      ALLMODULES   {set allmodules $v}
      ALLCHANNELS  {set allchannels $v}

      ACTIONS  {
         set action [lindex $actionlist $i]
         eval [second $action]
      }

      READA       {read_fifo A}
      READB       {read_fifo B}
      READC       {read_fifo C}
      READD       {read_fifo D}

      DPP*   - 
      ADC*        {click-widget $w $v}

      ENGINEERING       {click-$v}

      default           {
           set p "click_[lowercase $w]"
           if {[info procs $p] == "$p"} then {$p} else {insert-debug "$arg"}
      }
   }
}

proc make-frame {} {
  global env actionlist module maxmodule modulelist maxadc rates
   global allchannels allmodules

set x 12c

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/vme_module" \
      :icon-name "[self icon-title]" :resize F

   :panel MODULE :layout T :padx 10 :pady 10
      :layout H; :row-gap 10; :col-gap 10

      :number MODULE-NUM  :label "Module number:" :min 1 :max $maxmodule :w 5 :value $module
      :menubutton MODULE-SEL :label "select:" :strings $modulelist

      :hskip 100
      :checkbox ALLCHANNELS :label "Act on all channels" :value $allchannels
      :hskip 50
      :checkbox ALLMODULES  :label "Act on all modules"  :value $allmodules

    foreach ch {A B C D} {

      :next-row T
      :message M$ch :label "Channel $ch" :bold T
      :next-row T

      :text DATA$ch :label "Data FIFO" :w 8  :valuex $x :deaf T
      :button READ$ch :label "Read"
      :next-row T

      :text DPP[set ch]CSR  :label "DPP CSR"  :w 8 :valuex $x
      :text DPP[set ch]WPTR :label "DPP Write Ptr"  :w 8 :valuex $x
      :text DPP[set ch]RPTR :label "DPP Read Ptr"  :w 8 :valuex $x
      :text DPP[set ch]4    :label "DPP Version" :w 8 :valuex $x
      :text DPP[set ch]5    :label "DPP Reg 5" :w 8 :valuex $x
      :text DPP[set ch]6    :label "DPP Reg 6" :w 8 :valuex $x
      :next-row T

      :text ADC[set ch]CTRL     :label "ADC CTRL"  :w 8 :valuex $x
      :text ADC[set ch]PRETRIG  :label "ADC Pretrig"  :w 8 :valuex $x
      :text ADC[set ch]HEADER   :label "ADC Header"  :w 8 :valuex $x
      :text ADC[set ch]TRIGGER  :label "ADC Trigger"  :w 8 :valuex $x
      :text ADC[set ch]3        :label "ADC Version" :w 8 :valuex $x
      :text ADC[set ch]4        :label "ADC Spare 4" :w 8 :valuex $x
    }

   :panel CLICK :layout T :padx 10
      :layout H; :col-gap 10

      :button REDISPLAY   :command click-redisplay
      :menubutton ENGINEERING :label "Engineering"  :menu [list software registers]

   :show T
}
